package tae.util;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * Created by guangling.hjh on 15-3-11.
 */
public class SignUtil {
    private static final Logger logger = Logger.getRootLogger();
    private static final String HEADER_PARAM_PREFIX = "header_";
    private static final String TOP_SIGN_LIST = "top_sign_list";

    private static final String SECRET = ""; //应用secret

    /**
     * @param headerMap http请求中header中的参数，包括”top_sign_list“域，这个域中用”,"连接了所有需要参与sign的header参数；
     *                  如果某参数没有值，也需要放入headerMap中，相应的values为""
     * @param queryMap  http请求中query中的参数
     * @param body      从http请求中取出的输入流(可参考getBody方法）
     * @param secret    签名密钥
     * @param encode    服务的编码方式（在配置服务url的页面中可以看到)
     * @return
     * @throws Exception
     */
    public static String sign(Map<String, String> headerMap, Map<String, String> queryMap, String body, String secret, String encode) throws Exception {

        //取出header中的top_sign_list，它的内容是用","隔开的参数名，这些参数需要参与sign
        //这么做的原因是header有可能会被改写
        String topSignList = headerMap.get(TOP_SIGN_LIST);
        if (StringUtils.isNotBlank(topSignList)) {
            String[] headerSignParams = StringUtils.split(topSignList, ",");

            for (int i = 0; i < headerSignParams.length; i++) {
                String headerSignParam = headerSignParams[i];

                String value = headerMap.get(headerSignParam);
                //为了避免与query中的参数同名，加上前辍
                queryMap.put(HEADER_PARAM_PREFIX + headerSignParam, value);
            }

        }

        String sign = signatureForSpi(queryMap, secret, true, false, body, encode);
        return sign;
    }

    public static String getBody(HttpServletRequest request) {
        InputStream inputStream = null;
        try {
            inputStream = request.getInputStream();
            String dataOrigin = IOUtils.toString(inputStream);

            return dataOrigin;
        } catch (Exception e) {
            logger.info("exception IOUtils.toString");
            throw new RuntimeException(e);
        }
    }

    private static String signatureForSpi(Map<String, String> params, String secret,
                                          boolean appendSecret, boolean isHMac, String body, String encode) throws Exception {
        StringBuilder sb = new StringBuilder();
        // append if not hmac
        if (!isHMac) {
            sb.append(secret);
        }
        if (params != null && !params.isEmpty()) {
            String[] names = params.keySet().toArray(ArrayUtils.EMPTY_STRING_ARRAY);
            Arrays.sort(names);
            for (int i = 0; i < names.length; i++) {
                String name = names[i];
                sb.append(name);
                sb.append(params.get(name));
            }
        }
        if (StringUtils.isNotBlank(body)) {
            sb.append(body);
        }
        if (appendSecret && !isHMac) {
            sb.append(secret);
        }
        String sign = null;
        try {
            //md5
            sign = DigestUtils.md5Hex(sb.toString().getBytes(encode)).toUpperCase();

        } catch (Exception e) {
            logger.info("exception DigestUtils.md5Hex");
            throw new RuntimeException(e);
        }
        return sign;
    }

    private static String date2ymdhms(Date date) {
        SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return f.format(date);
    }

    public static Boolean testSign(HttpServletRequest request) {
        Map<String, String> headerMap = getHeadersInfo(request);

        //获得 UrlDecode后的header和body参数
        Map<String, String> headers = new HashMap<String, String>();
        String body = null;
        try {
            if (headerMap != null && !headerMap.isEmpty()) {
                String[] headerParams = headerMap.keySet().toArray(ArrayUtils.EMPTY_STRING_ARRAY);

                for (int i = 0; i < headerParams.length; i++) {
                    String name = headerParams[i];
                    headers.put(name, URLDecoder.decode(headerMap.get(name), "UTF-8"));
                }
            }

            body = URLDecoder.decode(getBody(request), "UTF-8");
        } catch (UnsupportedEncodingException e) {
            logger.info(JSONObject.toJSONString(e.getStackTrace()));
        }

        //获得query参数，使用request.getParameterMap获得的参数有乱码
        Map<String, String> params = new HashMap<String, String>();
        String queryString = request.getQueryString();
        String[] param = StringUtils.split(queryString, "&");
        for (int i = 0; i < param.length; i++) {
            String[] kv = StringUtils.split(param[i], "=");
            if (kv.length == 2) {
                params.put(kv[0], kv[1]);
            }
        }
        //Map<String, String[]> params = request.getParameterMap();
        Map<String, String> queryMap = new HashMap<String, String>();
        String sign = "";

        sign = params.get("sign");
        logger.info("sign : " + sign);

        decode("item_id",false,params,queryMap);
        decode("sku_id",false,params,queryMap);
        decode("seller_nick",true,params,queryMap);
        decode("mix_buyer_nick",false,params,queryMap);
        decode("timestamp",false,params,queryMap);

        logger.info("queryMap : " + JSONObject.toJSONString(queryMap));

        //签名
        String sign2 = "";
        try {
            sign2 = sign(headers, queryMap, body, SECRET, "UTF-8");
        } catch (Exception e) {
            logger.info(JSONObject.toJSONString(e.getStackTrace()));
        }
        logger.info("sign2 : " + sign2);

        return sign2.equals(sign);
    }

    private static void decode(String pName, Boolean needDecode, Map<String, String> src, Map<String, String> dest) {
        if (StringUtils.isBlank(pName)) {
            return;
        }
        String pValue = src.get(pName);
        if (StringUtils.isBlank(pValue)) {
            dest.put(pName, "");
            return;
        }
        if (needDecode) {
            try {
                pValue = URLDecoder.decode(pValue, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                logger.info(JSONObject.toJSONString(e.getStackTrace()));
            }
        }
        dest.put(pName, pValue);
        return;
    }

    private static Map<String, String> getHeadersInfo(HttpServletRequest request) {

        Map<String, String> map = new HashMap<String, String>();

        Enumeration headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String key = (String) headerNames.nextElement();
            String value = request.getHeader(key);
            map.put(key, value);
        }

        return map;
    }
}